Type: Package
Package: natcpp
Title: Fast C++ Primitives for the 'NeuroAnatomy Toolbox'
Version: 0.2
Authors@R: 
    person(given = "Gregory",
           family = "Jefferis",
           role = c("aut", "cre"),
           email = "jefferis@gmail.com",
           comment = c(ORCID = "0000-0002-0587-9355"))
Description: Fast functions implemented in C++ via 'Rcpp' to support the
    'NeuroAnatomy Toolbox' ('nat') ecosystem. These functions provide large
    speed-ups for basic manipulation of neuronal skeletons over pure R
    functions found in the 'nat' package. The expectation is that end
    users will not use this package directly, but instead the 'nat'
    package will automatically use routines from this package when it is
    available to enable large performance gains.
License: GPL (>= 3)
URL: https://github.com/natverse/natcpp, https://natverse.org/natcpp/
BugReports: https://github.com/natverse/natcpp/issues
Imports: Rcpp (>= 1.0.6)
Suggests: spelling, testthat (>= 3.0.0)
LinkingTo: Rcpp
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-10-14 21:32:26 UTC; jefferis
Author: Gregory Jefferis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0587-9355>)
Maintainer: Gregory Jefferis <jefferis@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-14 21:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 20:42:33 UTC; unix
Archs: natcpp.so.dSYM

Type: Package
Package: natcpp
Title: Fast C++ Primitives for the 'NeuroAnatomy Toolbox'
Version: 0.3.1
Authors@R: 
    person(given = "Gregory",
           family = "Jefferis",
           role = c("aut", "cre"),
           email = "jefferis@gmail.com",
           comment = c(ORCID = "0000-0002-0587-9355"))
Description: Fast functions implemented in C++ via 'Rcpp' to support the
    'NeuroAnatomy Toolbox' ('nat') ecosystem. These functions provide large
    speed-ups for basic manipulation of neuronal skeletons over pure R
    functions found in the 'nat' package. The expectation is that end
    users will not use this package directly, but instead the 'nat'
    package will automatically use routines from this package when it is
    available to enable large performance gains.
License: GPL (>= 3)
URL: https://github.com/natverse/natcpp, https://natverse.org/natcpp/
BugReports: https://github.com/natverse/natcpp/issues
Imports: Matrix, methods, Rcpp (>= 1.0.6)
Suggests: spelling, testthat (>= 3.0.0)
LinkingTo: Rcpp, RcppThread
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-20 13:56:19 UTC; jefferis
Author: Gregory Jefferis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0587-9355>)
Maintainer: Gregory Jefferis <jefferis@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-20 17:10:13 UTC

# Generated by roxygen2: do not edit by hand

export(c_EdgeListFromSegList)
export(c_ListofMatrixRows)
export(c_coords21dindex)
export(c_ijkpos)
export(c_listlengths)
export(c_seglengths)
export(c_sub2ind)
export(c_topntail)
export(c_topntail_list)
export(c_total_cable)
import(Rcpp)
useDynLib(natcpp, .registration = TRUE)

# natcpp

<!-- badges: start -->
[![natverse](https://img.shields.io/badge/natverse-Part%20of%20the%20natverse-a241b6)](https://natverse.org)
[![Docs](https://img.shields.io/badge/docs-100%25-brightgreen.svg)](https://natverse.org/natcpp/reference/)
[![R-CMD-check](https://github.com/natverse/natcpp/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/natverse/natcpp/actions/workflows/R-CMD-check.yaml)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![CRAN status](https://www.r-pkg.org/badges/version/natcpp)](https://CRAN.R-project.org/package=natcpp)
[![Codecov test coverage](https://codecov.io/gh/natverse/natcpp/graph/badge.svg)](https://app.codecov.io/gh/natverse/natcpp)
<!-- badges: end -->

The goal of *natcpp* is to provide accelerated routines through compiled C++ 
code for basic functions to support the [natverse](https://natverse.org) 
computational neuroanatomy ecosystem.
*natcpp* is not designed to be used by end users but instead the 
[nat](https://natverse.org/nat/) package will use it when available to enable
large speed-ups for some primitive operations. End users should therefore not 
count on the  *natcpp* functions/interface being stable, especially during its
initial development.

Our plan is to get this onto
CRAN quite quickly so that binary packages will be available to Mac/Windows 
users who do not have a full C compiler toolchain installed. Once on CRAN, 
we will try to keep the package simple and stable, with infrequent
updates. A development version will be available via the relevant github
repository (see below). 
We do not intend to specify a GitHub remote so these updates will not be 
installed by default i.e. we will prefer CRAN binaries unless the user
intervenes.

## Installation

Install the released version of natcpp from [CRAN](https://cran.r-project.org/package=natcpp) with:

``` r
install.packages("natcpp")
```

For the development version, please do:

``` r
install.packages("natmanager")
natmanager::install(pkgs = 'natverse/natcpp')
```


## Example

As already noted, the intention is that end users will only use the nat package
(and that will automagically use natcpp when available). However to prove that
everything is set up properly, you could do:

``` r
library(nat)
library(natcpp)
topntail(as.seglist(Cell07PNs[[1]]))

```

#' Sparse weighted Jaccard similarity via C++
#'
#' Compute the weighted Jaccard similarity matrix for a
#' \link[Matrix:dgCMatrix-class]{dgCMatrix}, returning a
#' sparse result. Uses \code{weighted_jaccard_sparse_fill} to compute min-sums
#' only for column (or row) pairs that share at least one non-zero feature, then
#' normalises to similarity. Only the upper triangle is computed, taking
#' advantage of the symmetry of the Jaccard index.
#'
#' @param x A \link[Matrix:dgCMatrix-class]{dgCMatrix} (sparse column-compressed matrix)
#' @param transpose If \code{FALSE} (default), compare columns; if
#'   \code{TRUE}, compare rows.
#' @param display_progress Whether to show a text progress bar (default
#'   \code{TRUE}).
#' @param threads Number of threads for parallel computation (default 4).
#'   Set to 0 to use all available cores.
#' @param triangle If \code{TRUE}, return a symmetric \code{dsCMatrix}
#'   (upper triangle only). If \code{FALSE} (default), return a general
#'   \link[Matrix:dgCMatrix-class]{dgCMatrix}.
#' @param distance If \code{TRUE}, return distance (\code{1 - similarity})
#'   instead of similarity. Default \code{FALSE}. A warning is issued since
#'   sparse distance matrices are typically dense.
#' @return A sparse similarity (or distance) matrix: \code{dsCMatrix} when
#'   \code{triangle = TRUE}, \link[Matrix:dgCMatrix-class]{dgCMatrix} otherwise.
#' @importFrom methods as
#' @export
#' @seealso \code{\link{c_weighted_jaccard_dense}} for the dense equivalent
#' @examples
#' \dontrun{
#' library(Matrix)
#' m <- sparseMatrix(i = c(1,2,1,2,3,3), j = c(1,1,2,2,2,3),
#'                   x = c(4,2,1,3,3,1), dims = c(3,3))
#' c_weighted_jaccard_sparse(m)
#' }
c_weighted_jaccard_sparse <- function(x, transpose = FALSE, display_progress = TRUE,
                                      threads = 4L, triangle = FALSE,
                                      distance = FALSE) {
  if (distance)
    warning("distance=TRUE with sparse output produces a mostly-dense matrix; ",
            "consider using c_weighted_jaccard_dense() with triangle=TRUE instead.")

  crossfun <- if (transpose) Matrix::tcrossprod else Matrix::crossprod
  n <- if (transpose) nrow(x) else ncol(x)

  if (length(x@x) == 0L) {
    sim_val <- if (distance) 0 else 1
    m <- Matrix::sparseMatrix(i = seq_len(n), j = seq_len(n), x = sim_val,
                              dims = c(n, n), symmetric = triangle)
    if (!triangle)
      m <- as(m, "generalMatrix")
    return(m)
  }

  # Sparsity pattern from binarised crossprod — dsCMatrix (upper triangle)
  b <- x
  b@x <- rep(1, length(b@x))
  A <- crossfun(b)

  # Column/row totals for denominator
  totals <- if (transpose) Matrix::rowSums(x) else Matrix::colSums(x)

  # Fill in min_sums using C++ (only upper triangle entries)
  A@x <- weighted_jaccard_sparse_fill(x, A, transpose = transpose,
                                      display_progress = display_progress,
                                      threads = threads)

  # Convert min_sums to similarity: sim = ms / (s_i + s_j - ms)
  col_idx <- rep(seq_along(diff(A@p)), diff(A@p))
  row_idx <- A@i + 1L
  denom <- totals[row_idx] + totals[col_idx] - A@x
  nonzero <- denom != 0
  A@x[nonzero] <- A@x[nonzero] / denom[nonzero]
  A@x[!nonzero] <- 0

  # Set diagonal to 1 for similarity
  diag_pos <- which(A@i + 1L == col_idx)
  A@x[diag_pos] <- 1

  if (distance) {
    A@x <- 1 - A@x  # diagonal 1→0, off-diag sim→1-sim
  }

  if (!triangle)
    A <- as(A, "generalMatrix")

  A
}

test_that("c_weighted_jaccard_dense computes correct similarity", {
  m <- Matrix::sparseMatrix(
    i = c(1L, 2L, 1L, 2L, 3L, 3L),
    j = c(1L, 1L, 2L, 2L, 2L, 3L),
    x = c(4, 2, 1, 3, 3, 1),
    dims = c(3L, 3L)
  )

  # Naive reference: sim[a,b] = sum(pmin(col_a, col_b)) / sum(pmax(col_a, col_b))
  dm <- as.matrix(m)
  n <- ncol(dm)
  ref <- matrix(0, n, n)
  for (a in seq_len(n)) {
    for (b in seq_len(n)) {
      mins <- sum(pmin(dm[, a], dm[, b]))
      maxs <- sum(pmax(dm[, a], dm[, b]))
      ref[a, b] <- if (maxs > 0) mins / maxs else 0
    }
  }
  diag(ref) <- 1

  sim <- c_weighted_jaccard_dense(m, transpose = FALSE)
  expect_equal(sim, ref, tolerance = 1e-12)
})

test_that("c_weighted_jaccard_dense transpose works", {
  m <- Matrix::sparseMatrix(
    i = c(1L, 2L, 1L, 2L, 3L, 3L),
    j = c(1L, 1L, 2L, 2L, 2L, 3L),
    x = c(4, 2, 1, 3, 3, 1),
    dims = c(3L, 3L)
  )

  dm <- as.matrix(m)
  nr <- nrow(dm)
  ref_t <- matrix(0, nr, nr)
  for (a in seq_len(nr)) {
    for (b in seq_len(nr)) {
      mins <- sum(pmin(dm[a, ], dm[b, ]))
      maxs <- sum(pmax(dm[a, ], dm[b, ]))
      ref_t[a, b] <- if (maxs > 0) mins / maxs else 0
    }
  }
  diag(ref_t) <- 1

  sim_t <- c_weighted_jaccard_dense(m, transpose = TRUE)
  expect_equal(sim_t, ref_t, tolerance = 1e-12)
})

test_that("c_weighted_jaccard_sparse matches dense", {
  m <- Matrix::sparseMatrix(
    i = c(1L, 2L, 1L, 2L, 3L, 3L),
    j = c(1L, 1L, 2L, 2L, 2L, 3L),
    x = c(4, 2, 1, 3, 3, 1),
    dims = c(3L, 3L)
  )

  sim_dense <- c_weighted_jaccard_dense(m, transpose = FALSE)
  sim_sparse <- c_weighted_jaccard_sparse(m, transpose = FALSE, display_progress = FALSE)
  expect_equal(as.matrix(sim_sparse), sim_dense, tolerance = 1e-12)

  sim_dense_t <- c_weighted_jaccard_dense(m, transpose = TRUE)
  sim_sparse_t <- c_weighted_jaccard_sparse(m, transpose = TRUE, display_progress = FALSE)
  expect_equal(as.matrix(sim_sparse_t), sim_dense_t, tolerance = 1e-12)
})

test_that("c_weighted_jaccard_dense triangle returns dist object", {
  m <- Matrix::sparseMatrix(
    i = c(1L, 2L, 1L, 2L, 3L, 3L),
    j = c(1L, 1L, 2L, 2L, 2L, 3L),
    x = c(4, 2, 1, 3, 3, 1),
    dims = c(3L, 3L)
  )

  full <- c_weighted_jaccard_dense(m)

  # triangle=TRUE returns a dist object
  tri <- c_weighted_jaccard_dense(m, triangle = TRUE)
  expect_s3_class(tri, "dist")
  expect_equal(attr(tri, "Size"), ncol(m))

  # triangle similarity matches as.dist of full similarity
  # (as.dist extracts lower triangle; dist stores 1-sim by convention,
  #  but here we just compare the raw values)
  ref_lower <- full[lower.tri(full)]
  expect_equal(as.numeric(tri), ref_lower, tolerance = 1e-12)

  # triangle + distance matches as.dist(1 - full)
  tri_d <- c_weighted_jaccard_dense(m, triangle = TRUE, distance = TRUE)
  expect_s3_class(tri_d, "dist")
  expect_equal(as.numeric(tri_d), 1 - ref_lower, tolerance = 1e-12)
})

test_that("c_weighted_jaccard_dense distance works for full matrix", {
  m <- Matrix::sparseMatrix(
    i = c(1L, 2L, 1L, 2L, 3L, 3L),
    j = c(1L, 1L, 2L, 2L, 2L, 3L),
    x = c(4, 2, 1, 3, 3, 1),
    dims = c(3L, 3L)
  )

  full <- c_weighted_jaccard_dense(m)
  full_d <- c_weighted_jaccard_dense(m, distance = TRUE)
  expect_equal(full_d, 1 - full, tolerance = 1e-12)
})

test_that("c_weighted_jaccard_sparse triangle and distance args", {
  m <- Matrix::sparseMatrix(
    i = c(1L, 2L, 1L, 2L, 3L, 3L),
    j = c(1L, 1L, 2L, 2L, 2L, 3L),
    x = c(4, 2, 1, 3, 3, 1),
    dims = c(3L, 3L)
  )

  # Default: dgCMatrix
  sp <- c_weighted_jaccard_sparse(m, display_progress = FALSE)
  expect_true(inherits(sp, "dgCMatrix"))

  # triangle=TRUE: dsCMatrix
  sp_tri <- c_weighted_jaccard_sparse(m, display_progress = FALSE, triangle = TRUE)
  expect_true(inherits(sp_tri, "dsCMatrix"))
  expect_equal(as.matrix(sp_tri), as.matrix(sp), tolerance = 1e-12)

  # distance=TRUE warns
  expect_warning(
    sp_d <- c_weighted_jaccard_sparse(m, display_progress = FALSE, distance = TRUE),
    "distance=TRUE"
  )
  # Stored entries are correct (1-sim); structural zeros remain 0 instead of 1
  # — this is the inherent limitation the warning describes
  full <- c_weighted_jaccard_dense(m)
  ref_d <- 1 - full
  ref_d[full == 0 & row(full) != col(full)] <- 0  # zero out unstored positions
  expect_equal(as.matrix(sp_d), ref_d, tolerance = 1e-12)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Compute summed segment lengths or total cable
#' @description \code{c_seglengths} computes the summed segment length equivalent
#'  to \code{nat::seglengths(sumsegment = T)}
#' @param sl A \code{seglist} with 1-indices into vectors x,y,z
#' @param x,y,z Numeric vectors with 3D coordinate data
#' (which could be columns from a data frame)
#' @export
c_seglengths <- function(sl, x, y, z) {
    .Call(`_natcpp_c_seglengths`, sl, x, y, z)
}

#' @description \code{c_total_cable} computes the summed total cable for a
#' whole neuron. It's intended use is the \code{nat::summary.neuron} function.
#' @rdname c_seglengths
#' @export
c_total_cable <- function(sl, x, y, z) {
    .Call(`_natcpp_c_total_cable`, sl, x, y, z)
}

#' Convert physical coordinates to pixel coordinates
#'
#' @param xyz Nx3 matrix of physical coordinates
#' @param dims Integer dimensions of the 3d image array
#' @param origin Numeric: 3d coordinates of the origin
#' @param voxdims Numeric: 3 numbers describing the voxel dimensions
#' @param clamp Logical: whether or not to clamp values within the pixel
#'   boundaries of the image.
#' @return Nx3 integer matrix of pixel coordinates
#' @export
c_ijkpos <- function(xyz, origin, voxdims, dims, clamp = FALSE) {
    .Call(`_natcpp_c_ijkpos`, xyz, origin, voxdims, dims, clamp)
}

#' Find 1D index given n-dimensional indices
#' @param dims Integer dimensions of the array (usually 3d)
#' @param indices Nx3 integer matrix of pixel coordinates
#' @return numeric vector of linear indices into the array
#' @export
c_sub2ind <- function(dims, indices) {
    .Call(`_natcpp_c_sub2ind`, dims, indices)
}

#' Convert physical coordinates to 1d indices into image array
#'
#' @param xyz Nx3 matrix or data.frame of physical coordinates
#' @param dims Integer dimensions of the 3d image array
#' @param origin Numeric: 3d coordinates of the origin
#' @param voxdims Numeric: 3 numbers describing the voxel dimensions
#' @param clamp Logical: whether or not to clamp values within the pixel
#'   boundaries of the image.
#' @return Nx3 integer matrix of pixel coordinates
#' @export
c_coords21dindex <- function(xyz, origin, voxdims, dims, clamp = FALSE) {
    .Call(`_natcpp_c_coords21dindex`, xyz, origin, voxdims, dims, clamp)
}

#' Convert a matrix into list of row vectors
#'
#' @details Typically this will be for 3D coordinates but there are no limits
#'   on row length.
#' @param object An integer, numeric, character or logical matrix of N rows and
#'   M columns
#' @return a list containing N vectors of length M corresponding to the rows of
#'   \code{object}.
#' @export
#' @examples
#' \dontrun{
#' library(nat)
#' xyz=xyzmatrix(Cell07PNs)
#' mat2list = function(m) {
#' um=unname(m)
#' lapply(1:nrow(um), function(i) um[i,])
#' }
#' bench::mark(rcpp=c_ListofMatrixRows(xyz), r=mat2list(xyz))
#' }
#' @export
c_ListofMatrixRows <- function(object) {
    .Call(`_natcpp_c_ListofMatrixRows`, object)
}

#' A simple function to compute the lengths of the elements of an R list
#'
#' @details This is equivalent to the \code{base::lengths} however it it much
#' faster for long lists (and somewhat slower for short ones).
#' @param L a list
#' @return An integer vector containing the length of each element of \code{L}
#' @export
c_listlengths <- function(L) {
    .Call(`_natcpp_c_listlengths`, L)
}

#' Find the first and last elements of all vectors in a list
#'
#' @description \code{c_topntail} returns an 2xN matrix containing the start and end
#' of each of the vectors in the input list. Length 0 vectors are ignored, while
#' length 1 vectors are duplicated
#'
#' @param L a list containing integer vectors, typically a \code{seglist}
#' @return For \code{c_topntail} an integer \code{matrix}. For \code{c_topntail_list}
#'   a \code{list}.
#' @export
c_topntail <- function(L) {
    .Call(`_natcpp_c_topntail`, L)
}

#' @description For \code{c_topntail_list}, a list of the same length as
#' \code{L} having the same elements when their length is <=2 or
#' the first and last elements when length>2.
#' @export
#' @rdname c_topntail
c_topntail_list <- function(L) {
    .Call(`_natcpp_c_topntail_list`, L)
}

#' Turn a segment list into an edgelist suitable for constructing an ngraph
#' @details It is up to the caller to generate the \code{seglist}.
#' Note that isolated points will be dropped since they have no edges.
#' @param L a list containing integer vectors from \code{as.seglist}
#' @return An integer matrix of N rows and 2 columns
#' @export
#' @examples
#'
#' \dontrun{
#' library(nat)
#' # make a neuron with multiple subtrees
#' n=prune_vertices(Cell07PNs[[1]], 48L)
#' # Must use flatten=T if including all subtrees
#' sl=as.seglist(n, all = TRUE, flatten = TRUE)
#' c_EdgeListFromSegList(sl)
#' }
c_EdgeListFromSegList <- function(L) {
    .Call(`_natcpp_c_EdgeListFromSegList`, L)
}

#' @noRd
weighted_jaccard_sparse_fill <- function(x, pattern, transpose = FALSE, display_progress = TRUE, threads = 4L) {
    .Call(`_natcpp_weighted_jaccard_sparse_fill`, x, pattern, transpose, display_progress, threads)
}

#' Dense weighted Jaccard similarity via C++
#'
#' Compute the full weighted Jaccard similarity matrix for a
#' \link[Matrix:dgCMatrix-class]{dgCMatrix} using
#' an adaptive dense accumulation strategy. For small output matrices, uses a
#' feature-oriented loop; for large outputs, switches to a column-oriented
#' loop for better cache performance.
#'
#' @param x A \link[Matrix:dgCMatrix-class]{dgCMatrix} (sparse column-compressed matrix)
#' @param transpose If \code{FALSE}, compare columns; if \code{TRUE}, compare
#'   rows
#' @param threads Number of threads (default 4). Set to 0 for all cores.
#' @param triangle If \code{TRUE}, return only the lower triangle as a flat
#'   numeric vector in \code{\link{dist}} layout. If \code{FALSE} (default),
#'   return a full square matrix.
#' @param distance If \code{TRUE}, return distance (\code{1 - similarity})
#'   instead of similarity. Default \code{FALSE}.
#' @return A dense numeric similarity matrix, or a numeric vector in
#'   \code{\link{dist}} layout when \code{triangle = TRUE}.
#' @export
c_weighted_jaccard_dense <- function(x, transpose = FALSE, threads = 4L, triangle = FALSE, distance = FALSE) {
    .Call(`_natcpp_c_weighted_jaccard_dense`, x, transpose, threads, triangle, distance)
}

test_that("multiplication works", {
  m=matrix(1:6, ncol=3, byrow = T)
  l=list(1:3, 4:6)
  colnames(m)=c("X","Y","Z")
  expect_equal(c_ListofMatrixRows(m), l)

  nm=matrix(rnorm(n = 300), ncol=3)

  expect_equal(c_ListofMatrixRows(nm),
               lapply(seq_len(nrow(nm)), function(i) nm[i,]))

  expect_equal(c_ListofMatrixRows(matrix(1L, nrow=2, ncol=0)),
               list(integer(), integer()))
  expect_equal(c_ListofMatrixRows(matrix(1L, nrow=0, ncol=2)),
               list())
  expect_error(c_ListofMatrixRows(l))
  # matrix of lists
  ml=matrix(rep(list(1), 4), ncol = 2)
  expect_error(c_ListofMatrixRows(ml))
})

test_that("c_EdgeListFromSegList works", {
  n=readRDS(test_path('testdata/cell071.rds'))
  # el=nat:::EdgeListFromSegList(n$SegList)
  # mode(el)='integer'
  # dimnames(el)=NULL
  # expect_snapshot_value(el, style = 'json2')
  expect_snapshot_value(c_EdgeListFromSegList(n$SegList), style = 'json2')
})


test_that("topntail works", {
  n=readRDS(test_path('testdata/cell071.rds'))
  # slt=sapply(n$SegList, function(x) as.integer(c(x[1], x[length(x)])))
  # expect_snapshot_value(slt, style = 'json2')
  expect_snapshot_value(tntm <- c_topntail(n$SegList), style = 'json2')
  expect_equal(c_topntail_list(n$SegList), apply(tntm, 2, c, simplify = F))
})

test_that("ijkpos, c_sub2ind, c_coords21dindex", {
  origin  <- c(10, 20, 30)
  voxdims <- c(1, 2, 3)
  dims    <- c(20, 30, 40)

  xyz_mat <- matrix(c(10, 20, 30,
                      11, 22, 33,
                      40, 80, 160), ncol = 3, byrow = TRUE)
  xyz_df <- data.frame(x = c(10, 11),
                       y = c(20, 22),
                       z = c(30, 33))
  bl=structure(c(1L, 2L, 20L, 1L, 2L, 30L, 1L, 2L, 40L), dim = c(3L, 3L))

  expect_warning(expect_equal(
    c_ijkpos(xyz_mat, origin, voxdims, dims, clamp = TRUE),
    bl))

  expect_silent(ijk <- c_ijkpos(xyz_mat[1:2,], origin, voxdims, dims, clamp = TRUE))
  expect_equal(c_ijkpos(xyz_mat[1:2,], origin, voxdims, dims, clamp = TRUE),
               c_ijkpos(xyz_df, origin, voxdims, dims, clamp = TRUE))

  bl=c(1,622)
  expect_equal(c_sub2ind(dims, indices = ijk), bl)
  expect_error(c_sub2ind(dims[1:2], indices = ijk))

  # test that we can use double indices as input
  mode(ijk)='double'
  expect_equal(
    c_sub2ind(dims, indices = ijk),
    c_coords21dindex(xyz_mat[1:2,], origin = origin, voxdims = voxdims, dims = dims))
  expect_warning(expect_equal(
    c_coords21dindex(as.data.frame(xyz_mat), origin = origin, voxdims = voxdims, dims = dims, clamp = T),
    c(1, 622, 24000)))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_jaccard.R
\name{c_weighted_jaccard_sparse}
\alias{c_weighted_jaccard_sparse}
\title{Sparse weighted Jaccard similarity via C++}
\usage{
c_weighted_jaccard_sparse(
  x,
  transpose = FALSE,
  display_progress = TRUE,
  threads = 4L,
  triangle = FALSE,
  distance = FALSE
)
}
\arguments{
\item{x}{A \link[Matrix:dgCMatrix-class]{dgCMatrix} (sparse column-compressed matrix)}

\item{transpose}{If \code{FALSE} (default), compare columns; if
\code{TRUE}, compare rows.}

\item{display_progress}{Whether to show a text progress bar (default
\code{TRUE}).}

\item{threads}{Number of threads for parallel computation (default 4).
Set to 0 to use all available cores.}

\item{triangle}{If \code{TRUE}, return a symmetric \code{dsCMatrix}
(upper triangle only). If \code{FALSE} (default), return a general
\link[Matrix:dgCMatrix-class]{dgCMatrix}.}

\item{distance}{If \code{TRUE}, return distance (\code{1 - similarity})
instead of similarity. Default \code{FALSE}. A warning is issued since
sparse distance matrices are typically dense.}
}
\value{
A sparse similarity (or distance) matrix: \code{dsCMatrix} when
  \code{triangle = TRUE}, \link[Matrix:dgCMatrix-class]{dgCMatrix} otherwise.
}
\description{
Compute the weighted Jaccard similarity matrix for a
\link[Matrix:dgCMatrix-class]{dgCMatrix}, returning a
sparse result. Uses \code{weighted_jaccard_sparse_fill} to compute min-sums
only for column (or row) pairs that share at least one non-zero feature, then
normalises to similarity. Only the upper triangle is computed, taking
advantage of the symmetry of the Jaccard index.
}
\examples{
\dontrun{
library(Matrix)
m <- sparseMatrix(i = c(1,2,1,2,3,3), j = c(1,1,2,2,2,3),
                  x = c(4,2,1,3,3,1), dims = c(3,3))
c_weighted_jaccard_sparse(m)
}
}
\seealso{
\code{\link{c_weighted_jaccard_dense}} for the dense equivalent
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_seglengths}
\alias{c_seglengths}
\alias{c_total_cable}
\title{Compute summed segment lengths or total cable}
\usage{
c_seglengths(sl, x, y, z)

c_total_cable(sl, x, y, z)
}
\arguments{
\item{sl}{A \code{seglist} with 1-indices into vectors x,y,z}

\item{x, y, z}{Numeric vectors with 3D coordinate data
(which could be columns from a data frame)}
}
\description{
\code{c_seglengths} computes the summed segment length equivalent
 to \code{nat::seglengths(sumsegment = T)}

\code{c_total_cable} computes the summed total cable for a
whole neuron. It's intended use is the \code{nat::summary.neuron} function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/natcpp-package.R
\docType{package}
\name{natcpp-package}
\alias{natcpp}
\alias{natcpp-package}
\title{natcpp: Fast C++ Primitives for the 'NeuroAnatomy Toolbox'}
\description{
Fast functions implemented in C++ via 'Rcpp' to support the 'NeuroAnatomy Toolbox' ('nat') ecosystem. These functions provide large speed-ups for basic manipulation of neuronal skeletons over pure R functions found in the 'nat' package. The expectation is that end users will not use this package directly, but instead the 'nat' package will automatically use routines from this package when it is available to enable large performance gains.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/natverse/natcpp}
  \item \url{https://natverse.org/natcpp/}
  \item Report bugs at \url{https://github.com/natverse/natcpp/issues}
}

}
\author{
\strong{Maintainer}: Gregory Jefferis \email{jefferis@gmail.com} (\href{https://orcid.org/0000-0002-0587-9355}{ORCID})

Authors:
\itemize{
  \item Gregory Jefferis \email{jefferis@gmail.com} (\href{https://orcid.org/0000-0002-0587-9355}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_listlengths}
\alias{c_listlengths}
\title{A simple function to compute the lengths of the elements of an R list}
\usage{
c_listlengths(L)
}
\arguments{
\item{L}{a list}
}
\value{
An integer vector containing the length of each element of \code{L}
}
\description{
A simple function to compute the lengths of the elements of an R list
}
\details{
This is equivalent to the \code{base::lengths} however it it much
faster for long lists (and somewhat slower for short ones).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_weighted_jaccard_dense}
\alias{c_weighted_jaccard_dense}
\title{Dense weighted Jaccard similarity via C++}
\usage{
c_weighted_jaccard_dense(
  x,
  transpose = FALSE,
  threads = 4L,
  triangle = FALSE,
  distance = FALSE
)
}
\arguments{
\item{x}{A \link[Matrix:dgCMatrix-class]{dgCMatrix} (sparse column-compressed matrix)}

\item{transpose}{If \code{FALSE}, compare columns; if \code{TRUE}, compare
rows}

\item{threads}{Number of threads (default 4). Set to 0 for all cores.}

\item{triangle}{If \code{TRUE}, return only the lower triangle as a flat
numeric vector in \code{\link{dist}} layout. If \code{FALSE} (default),
return a full square matrix.}

\item{distance}{If \code{TRUE}, return distance (\code{1 - similarity})
instead of similarity. Default \code{FALSE}.}
}
\value{
A dense numeric similarity matrix, or a numeric vector in
  \code{\link{dist}} layout when \code{triangle = TRUE}.
}
\description{
Compute the full weighted Jaccard similarity matrix for a
\link[Matrix:dgCMatrix-class]{dgCMatrix} using
an adaptive dense accumulation strategy. For small output matrices, uses a
feature-oriented loop; for large outputs, switches to a column-oriented
loop for better cache performance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_ListofMatrixRows}
\alias{c_ListofMatrixRows}
\title{Convert a matrix into list of row vectors}
\usage{
c_ListofMatrixRows(object)
}
\arguments{
\item{object}{An integer, numeric, character or logical matrix of N rows and
M columns}
}
\value{
a list containing N vectors of length M corresponding to the rows of
  \code{object}.
}
\description{
Convert a matrix into list of row vectors
}
\details{
Typically this will be for 3D coordinates but there are no limits
  on row length.
}
\examples{
\dontrun{
library(nat)
xyz=xyzmatrix(Cell07PNs)
mat2list = function(m) {
um=unname(m)
lapply(1:nrow(um), function(i) um[i,])
}
bench::mark(rcpp=c_ListofMatrixRows(xyz), r=mat2list(xyz))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_sub2ind}
\alias{c_sub2ind}
\title{Find 1D index given n-dimensional indices}
\usage{
c_sub2ind(dims, indices)
}
\arguments{
\item{dims}{Integer dimensions of the array (usually 3d)}

\item{indices}{Nx3 integer matrix of pixel coordinates}
}
\value{
numeric vector of linear indices into the array
}
\description{
Find 1D index given n-dimensional indices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_EdgeListFromSegList}
\alias{c_EdgeListFromSegList}
\title{Turn a segment list into an edgelist suitable for constructing an ngraph}
\usage{
c_EdgeListFromSegList(L)
}
\arguments{
\item{L}{a list containing integer vectors from \code{as.seglist}}
}
\value{
An integer matrix of N rows and 2 columns
}
\description{
Turn a segment list into an edgelist suitable for constructing an ngraph
}
\details{
It is up to the caller to generate the \code{seglist}.
Note that isolated points will be dropped since they have no edges.
}
\examples{

\dontrun{
library(nat)
# make a neuron with multiple subtrees
n=prune_vertices(Cell07PNs[[1]], 48L)
# Must use flatten=T if including all subtrees
sl=as.seglist(n, all = TRUE, flatten = TRUE)
c_EdgeListFromSegList(sl)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_topntail}
\alias{c_topntail}
\alias{c_topntail_list}
\title{Find the first and last elements of all vectors in a list}
\usage{
c_topntail(L)

c_topntail_list(L)
}
\arguments{
\item{L}{a list containing integer vectors, typically a \code{seglist}}
}
\value{
For \code{c_topntail} an integer \code{matrix}. For \code{c_topntail_list}
  a \code{list}.
}
\description{
\code{c_topntail} returns an 2xN matrix containing the start and end
of each of the vectors in the input list. Length 0 vectors are ignored, while
length 1 vectors are duplicated

For \code{c_topntail_list}, a list of the same length as
\code{L} having the same elements when their length is <=2 or
the first and last elements when length>2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{c_ijkpos}
\alias{c_ijkpos}
\title{Convert physical coordinates to pixel coordinates}
\usage{
c_ijkpos(xyz, origin, voxdims, dims, clamp = FALSE)
}
\arguments{
\item{xyz}{Nx3 matrix of physical coordinates}

\item{origin}{Numeric: 3d coordinates of the origin}

\item{voxdims}{Numeric: 3 numbers describing the voxel dimensions}

\item{dims}{Integer dimensions of the 3d image array}

\item{clamp}{Logical: whether or not to clamp values within the pixel
boundaries of the image.}
}
\value{
Nx3 integer matrix of pixel coordinates
}
\description{
Convert physical coordinates to pixel coordinates
}
